Sound Card Database Design
==========================

1. Overview
-----------

The database is used for storing data for every piece of
supported hardware. it contains autogenerated data
(like device and vendor ids, parameter description etc)
plus some manually added data (volume for nonstandard
mixer channels needed for proper card initialization etc).

The database is placed in /usr/share/YaST2/data/sndcards.ycp
file in installed system.

2. The Database Structure
-------------------------

The whole database is an ycp map, you just need to call 
SCR::Read(.target.ycp, "/usr/share/YaST2/data/sndcards.ycp")

The topmost map contains keys:

 1. "cards"
 2. "indices"
 3. "mod_idx"
 4. "modules"

2.1 "cards"
-----------

The value is a map, for each alsa kernel module (as key) it 
contains list of names of cards supported by this module.
This list is needed for manual sound card config in the
sound module.

The module name is substituted by the module index (just 
to save space, this policy is used in mode places in DB).
see 2.3 for details.

2.2 "indices"
-------------

The value is a nested map, this part is used for searching the 
appropriate module for device_id/vendor_id pair. When looking
up the module for a card, first check the map indexed by vendor_id.
The result is a map which indices are device_ids and the value
is index to `mod_idx` where is stored the module name (as string).

2.3 "mod_idx"
-------------

Leaves of `indices` data structure are very redundant, the module names are
replaced by their indices. Value of `mod_idx` is a map with numerical index as
key and the module name as value.

2.4 "modules"
-------------

This structure contains map with module name as key and a map of data
as value.

The keys:

    description (string):   used in 'manual card selection' dialog
    name (string):	    !optional. used in very special cases. see appendix A
			    usually the module name is used as a default.
    mixer (map):	    some cards have special channels that needs to be
			    unmuted/set when initialized.
			    key is channel name, value means volume for this channel.
    main_volume (string)    name of the 'Master' volume element. this channel is
			    used in 'volume' dialog. this is always 'Master' (default)
			    except some PPC where the name is ... (huh i forgot..
			    probably 'Master Digital' or similar)
    params (map):	    parameters for module. module name is key in this map,
			    value is map with other data:
	    allows (string) possible values for option (if known). this string
			    has special syntax, see appendix B for details
	    default (string) default value fot this option
	    descr (locale/list) human readable desrciption for the option. 
			    if descr is list, it is sformated.
	    dialog (string) widget to be used for setting this option (not used
			    currently)
	    base (string)   ? (not used currently)
	    
3. Database Autogeneration
--------------------------

The generator sources are in data/ subdirectory,
to rebuild the database, just do:

    make clean #optionally
    make
    make install

this will generate the DB and install it into the system.

**NOTE:** the generated file must not be changed, the changes will be lost
after the next regeneration! See Chapter 4. for details how to add changes
manually to the database.

**NOTE:** the database is built automatically during RPM package build,
it contains only data relevant to the architecture where the package was built.

3.1 Creating the DB
-------------------

The data are read from `snd-*.ko` files located in 
`/lib/modules/*/kernel/sound/` directory.

The data are extracted using `modinfo` tool. The collected data are then
merged with the additional static data (see the next chapter)
and stored into the target DB file.


4. Supplying Data Manually
--------------------------

These additional files are stored in data/ subdirectory:

    data_cards.yml          sound card names
    data_extra_id.yml       additional vendor/device ids
    data_mixer.yml          extra mixer settings

Appendix A
----------

'allows' module options syntax

    {0} stands for 0
    {0,Disabled} is same as {0} (no idea why this was invented...)
    {1,Enabled} same as {1} (ditto)
    {0,5} stands for range 0,1,2,3,4,5
    {0,6,2} stands for range with step 2: 0,2,4,6
    {atom1,atom2} union of atom1 & atom2 is allowed

